

class InfoOverlay {
    constructor(graph) {
        this.graph = graph;

        this.activeTabNav = null;
        this.activeTabContent = null;

        this.tabContentPages = {};
    }

    create() {
        const overlayDiv = this.createOverlayMainDiv();
        this.createOverlayElements(overlayDiv);
        this.createBottomMenu(overlayDiv)

        jQuery('#infoOverlayCloseButton').click(function () {
            jQuery('#infoOverlay').slideUp('fast');
        });

    }

    createBottomMenu(overlayNode) {

        const bottomContainerDiv = document.createElement('div');
        bottomContainerDiv.className = 'bottom-container';
        overlayNode.appendChild(bottomContainerDiv);

        const bottomContainerNavDiv = document.createElement('div');
        bottomContainerNavDiv.className = 'bottom-container-nav';
        bottomContainerDiv.appendChild(bottomContainerNavDiv);

        const bottomContainerLinkDiv = document.createElement('div');
        bottomContainerLinkDiv.className = 'bottom-container-links';
        bottomContainerDiv.appendChild(bottomContainerLinkDiv);

        for(const [cls, color] of Object.entries(this.graph.edgeColors)) {

            const navTab = document.createElement('div');
            navTab.className = 'bottom-container-nav-tab';
            navTab.innerText = cls.slice(0, 3);
            navTab.style = "background-color: " + color;
            navTab.edgeType = cls;
            //tablink.setAttribute("onclick", "openTab(event, " + linkType + "_id)"); //unusual function call
            bottomContainerNavDiv.appendChild(navTab);
            jQuery(navTab).click(event => this.openTab(event))

            const tabContent = document.createElement('div');
            tabContent.className = "bottom-container-tab-content";
            tabContent.id = cls + "_id_content";
            tabContent.style = "background-color: " + color;
            bottomContainerLinkDiv.appendChild(tabContent);
            this.tabContentPages[cls] = tabContent;
        }

        this.activeTabNav = bottomContainerNavDiv.firstChild;
        this.activeTabContent = bottomContainerLinkDiv.firstChild;

        this.activeTabContent.classList.add("active-tab-content");
        this.activeTabNav.classList.add("active-tab-nav");
        this.activeTabNav.innerText = this.activeTabNav.edgeType;

    }

    openTab(event) {
        const navTab = event.target;
        const cls = navTab.edgeType;

        this.activeTabNav.classList.remove("active-tab-nav");
        this.activeTabNav.innerText = this.activeTabNav.innerText.slice(0, 3);
        navTab.classList.add("active-tab-nav");
        navTab.innerText = cls;

        this.activeTabContent.classList.remove("active-tab-content");
        this.tabContentPages[cls].classList.add("active-tab-content");

        this.activeTabNav = navTab;
        this.activeTabContent = this.tabContentPages[cls];
    }

    createOverlayMainDiv() {
        const sceneNode = getCanvasDivNode();
        const overlayNode = document.createElement('div');
        overlayNode.id = 'infoOverlay'
        overlayNode.className = 'detail-view';
        //overlayNode.innerText = 'Hello there!';
        sceneNode.insertBefore(overlayNode, sceneNode.childNodes[2]);
        return overlayNode;
    }

    createOverlayElements(overlayNode) {
        const close = document.createElement('div');
        close.innerHTML = '<p>&#10006;</p>';
        close.id = 'infoOverlayCloseButton';
        close.className = 'close-button';
        overlayNode.appendChild(close);

        const infoArea = document.createElement('div');
        infoArea.className = 'detail-view-info-area';
        overlayNode.appendChild(infoArea);

        const topArea = document.createElement('div');
        topArea.className = 'detail-view-top-area';
        infoArea.appendChild(topArea);

        const nodeImage = document.createElement('img');
        nodeImage.id = 'infoOverlayImage';
        nodeImage.src = plugin_path + 'datasets/images/default.jpg';
        nodeImage.className = 'detail-view-image';
        topArea.appendChild(nodeImage);

        const headline = document.createElement('h2');
        headline.id = 'infoOverlayHeadline';
        headline.innerText = 'Default Text';
        headline.className = 'detail-view-headline';
        topArea.appendChild(headline);

        const textArea = document.createElement('div');
        textArea.className = 'detail-view-text-area';
        infoArea.appendChild(textArea);

        const description = document.createElement('p');
        description.id = 'infoOverlayDescription';
        description.innerText = 'Default Text'
        description.setAttribute("overflow-y", "scroll");
        textArea.appendChild(description);
    }

    updateInfoOverlay(node) {
        jQuery('#infoOverlayHeadline').text(node.name);
        if ('image' in node) {
            jQuery('#infoOverlayImage').attr('src', plugin_path + 'datasets/images/' + node.image);
        } else {
            jQuery('#infoOverlayImage').attr('src', plugin_path + 'datasets/images/default.jpg');
        }

        if ('description' in node) {
            jQuery('#infoOverlayDescription').text(node.description);
        } else {
            jQuery('#infoOverlayDescription').text('Default Text');
        }

        this.updateTabs(node);

        jQuery('#infoOverlay').slideDown('fast');
    }

    clearTabContentPages() {
        for (const page of Object.values(this.tabContentPages)) {
            jQuery(page).empty();
        }
    }

    createReference(target) {
        const linkDiv = document.createElement('div');      //and if so a new element is created
        linkDiv.className = "link-img";

        if ('image' in target) {
            const linkImage = document.createElement('img');
            linkImage.src = plugin_path + 'datasets/images/' + target.image;
            linkDiv.appendChild(linkImage);
        }

        jQuery(linkDiv).on('click', () => {
            this.graph.focusOnNode(target);
            this.updateInfoOverlay(target);
        });
        return linkDiv;
    }

    updateTabs(node){
        this.clearTabContentPages();

        for (const link of node.links) {
            const target = link.source == node ? link.target : link.source;
            const reference = this.createReference(target);
            this.tabContentPages[link.type].appendChild(reference);
        }
    }

}